package data;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
public class GsdPointData {
//	private static DecimalFormat TowDec = new DecimalFormat("00"); 
//	private static DecimalFormat FourDec = new DecimalFormat("0000"); 
	private static DecimalFormat SpeedFormat = new DecimalFormat("0.00"); 
	private static SimpleDateFormat DateFormat = new SimpleDateFormat("yyyy-MM-dd");
	private static SimpleDateFormat TimeFormat = new SimpleDateFormat("HH-mm-ss");
	private static int DEFAULT_GMT_PLAS_TIME = 9;
	//日付,時刻,緯度,経度,速度(km/s),高度
	public static String getColumn(String sep){
		StringBuilder sbuf = new StringBuilder();
		sbuf.append("日付");
		sbuf.append(sep);
		sbuf.append("時刻");
		sbuf.append(sep);
		sbuf.append("緯度");
		sbuf.append(sep);
		sbuf.append("経度");
		sbuf.append(sep);
		sbuf.append("速度(km/s)");
		sbuf.append(sep);
		sbuf.append("高度");
		return sbuf.toString();
	}
	//6=35419643,139312603,101804,250408,509,-1
	private static final Pattern dataPattern = Pattern.compile("\\s*[0-9]+=(-?[0-9]+),(-?[0-9]+),([0-9]+),([0-9]+),([0-9]+),(-?[0-9]+)\\s*");
	private static final DecimalFormat dateTimeDataFormat = new DecimalFormat("000000");
	public static GsdPointData parseLine(String Line){
		return parseLine(Line,DEFAULT_GMT_PLAS_TIME);
	}
	public static GsdPointData parseLine(String Line,int GMTPlasHur){
		Matcher mc = dataPattern.matcher(Line);
		if(mc.matches()){
			//経緯度切り出し
			//
			StringBuilder lat = new StringBuilder();
			int latCutPoint = mc.group(1).length() - 6; 
			String deg = mc.group(1).substring(0,latCutPoint);
			String min = mc.group(1).substring(latCutPoint);
			if(deg.startsWith("-")){
				lat.append("S");
			} else {
				lat.append("N");
			}
			lat.append(GPSPosition.degFomrat.format(Math.abs(Integer.parseInt(deg))));
			lat.append(".");
			double min_double = Double.parseDouble(min) / 10000.0;
			int min_int = (int)Math.floor(min_double);
			lat.append(GPSPosition.minFomrat.format(min_int));
			lat.append(".");
			double sec_double = (min_double - min_int) * 60.0;
			lat.append(GPSPosition.secondFomrat.format(sec_double));
			
			//			
			StringBuilder lon = new StringBuilder();
			int lonCutPoint = mc.group(2).length() - 6; 
			deg = mc.group(2).substring(0,lonCutPoint);
			min = mc.group(2).substring(lonCutPoint);
			if(deg.startsWith("-")){
				lon.append("W");
			} else {
				lon.append("E");
			}
			lon.append(GPSPosition.degFomrat.format(Math.abs(Integer.parseInt(deg))));
			lon.append(".");
			min_double = Double.parseDouble(min) / 10000.0;
			min_int = (int)Math.floor(min_double);
			lon.append(GPSPosition.minFomrat.format(min_int));
			lon.append(".");
			sec_double = (min_double - min_int) * 60.0;
			lon.append(GPSPosition.secondFomrat.format(sec_double));
			//
			double att = Double.parseDouble(mc.group(6)) / 10000.0;
			if(mc.group(6).equals("-1")){
				att = 0;
			}
			//
			GPSPosition p = new GPSPosition(lat.toString(),lon.toString(),att);
			//時間切り出し
			int timeValue = Integer.parseInt(mc.group(3));
			String formatedTime = dateTimeDataFormat.format(timeValue);
			int hourOfDay = Integer.parseInt(formatedTime.substring(0,2)) + GMTPlasHur;
			int minute = Integer.parseInt(formatedTime.substring(2,4));
			int second = Integer.parseInt(formatedTime.substring(4,6));

			//日付きり出し
			int dateValue = Integer.parseInt(mc.group(4));
			String formatedDate = dateTimeDataFormat.format(dateValue);
			
			int dayOfMonth = Integer.parseInt(formatedDate.substring(0,2));
			int month = Integer.parseInt(formatedDate.substring(2,4));
			int year = Integer.parseInt(formatedDate.substring(4,6)) + 2000 ;
			//Month値は0が１月
			GregorianCalendar t = new GregorianCalendar(year,month-1,dayOfMonth,hourOfDay,minute,second);
			//System.out.println(DateFormat.format(t.getTime()));
			
			
			//速度切り出し
			double s = Double.parseDouble(mc.group(5)) / 100.0;
			GsdPointData data = new GsdPointData(p,t,s);
			return data;
		} else {
			if(!Line.equals("")){
				System.err.println(Line + "はGsdPointDataのパターンにマッチしません");
			}
			return null;
		}
	}
	
	private GPSPosition location;
	private GregorianCalendar time;
	private double Speed;

	public GsdPointData(GPSPosition p,GregorianCalendar t,double s){
		this.location = p;
		this.time = t;
		this.Speed = s;
	}


	public GPSPosition getLocation() {
		return location;
	}

	public double getSpeed() {
		return Speed;
	}

	public GregorianCalendar getTime() {
		return time;
	}
	
	public String getDateByStringFormat(){
		return DateFormat.format(this.time.getTime());
	}
	public String getTimeByStringFormat(){
		return TimeFormat.format(this.time.getTime());
	}
	
	public String toString(String sep){
		StringBuilder sbuf = new StringBuilder();
		sbuf.append(this.getDateByStringFormat());
		sbuf.append(sep);
		sbuf.append(this.getTimeByStringFormat());
		sbuf.append(sep);
		sbuf.append(this.getLocation().get緯度());
		sbuf.append(sep);
		sbuf.append(this.getLocation().get経度());
		sbuf.append(sep);
		sbuf.append(SpeedFormat.format(this.Speed));
		sbuf.append(sep);
		sbuf.append(GPSPosition.heightFomrat.format(this.getLocation().getPositonByDoubleDegreeValue()[2]));
		return sbuf.toString();
	}

}

