package parser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.ArrayList;

import data.GsdPointData;

public class GsdFileParser {

	public static ArrayList<GsdPointData> parseFile(File datafile) throws IOException{
		ArrayList<GsdPointData> dataList = new ArrayList<GsdPointData>();
		BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(datafile)));
		String Line;
		while((Line = reader.readLine()) != null){
			GsdPointData data = GsdPointData.parseLine(Line);
			if(data != null){
				dataList.add(data);
				
			}
		}
		reader.close();
		return dataList;
	}
	
	public static ArrayList<ArrayList<GsdPointData>> cutListByTime(ArrayList<GsdPointData> baseList,int second){
		long milisec = second*1000;
		ArrayList<ArrayList<GsdPointData>> cutList = new ArrayList<ArrayList<GsdPointData>>();
		ArrayList<GsdPointData> tempList = null;
		long start = 0;
		for(GsdPointData data:baseList){
			if(data.getTime().getTime().getTime() - start > milisec){
				if(tempList != null){
					cutList.add(tempList);
				}
				tempList = new ArrayList<GsdPointData>();
			}
			start = data.getTime().getTime().getTime();
			tempList.add(data);
		}
		if(tempList != null){
			cutList.add(tempList);
		}
		return cutList;		
	}

	
	
	public static void splitData(File inputFile,int splitSecond) throws IOException{
		File outputDir = new File(inputFile.getParentFile().getPath()+ File.separator + inputFile.getName().substring(0,inputFile.getName().lastIndexOf(".")));
		splitData(inputFile,outputDir,splitSecond);
	}
	public static void splitData(File inputFile,File outputDir,int splitSecond) throws IOException{
		ArrayList<GsdPointData> dataList = GsdFileParser.parseFile(inputFile);
		ArrayList<ArrayList<GsdPointData>> cutlist = cutListByTime(dataList,splitSecond);
		outputDir.mkdirs();
		for(ArrayList<GsdPointData> list:cutlist){
			File outputFile = new File(outputDir.getPath() + File.separator + list.get(0).getDateByStringFormat()+"-"+list.get(0).getTimeByStringFormat()+".csv");
			PrintStream writer = new PrintStream(outputFile);
			writer.println(GsdPointData.getColumn(","));
			for(GsdPointData data:list){
				writer.println(data.toString(","));
			}
			writer.close();
		}
	}

}
